# Generated by ariadne-codegen
# Source: vals/graphql/

from pydantic import Field

from .base_model import BaseModel
from .fragments import QuestionAnswerPairFragment


class CreateQuestionAnswerPair(BaseModel):
    create_question_answer_pair: "CreateQuestionAnswerPairCreateQuestionAnswerPair" = (
        Field(alias="createQuestionAnswerPair")
    )


class CreateQuestionAnswerPairCreateQuestionAnswerPair(BaseModel):
    question_answer_pair: (
        "CreateQuestionAnswerPairCreateQuestionAnswerPairQuestionAnswerPair"
    ) = Field(alias="questionAnswerPair")


class CreateQuestionAnswerPairCreateQuestionAnswerPairQuestionAnswerPair(
    QuestionAnswerPairFragment
):
    pass


CreateQuestionAnswerPair.model_rebuild()
CreateQuestionAnswerPairCreateQuestionAnswerPair.model_rebuild()
