#import { ResultJsonElementTypeFields } from "../fragments/shared.graphql"

mutation EvaluateTestChecks(
  $testId: String!
  $llmOutput: String!
  $outputContext: GenericScalar
  $parameters: ParameterInputType
) {
  evaluateTestChecks(
    testId: $testId
    llmOutput: $llmOutput
    outputContext: $outputContext
    parameters: $parameters
  ) {
    resultJson {
      ...ResultJsonElementTypeFields
    }
  }
}
