# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import Optional

from pydantic import Field

from .base_model import BaseModel
from .fragments import RunFragment


class CreateEmptyRunObject(BaseModel):
    create_question_answer_set: Optional[
        "CreateEmptyRunObjectCreateQuestionAnswerSet"
    ] = Field(alias="createQuestionAnswerSet")


class CreateEmptyRunObjectCreateQuestionAnswerSet(BaseModel):
    run: Optional["CreateEmptyRunObjectCreateQuestionAnswerSetRun"]


class CreateEmptyRunObjectCreateQuestionAnswerSetRun(RunFragment):
    pass


CreateEmptyRunObject.model_rebuild()
CreateEmptyRunObjectCreateQuestionAnswerSet.model_rebuild()
