# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import List, Optional

from pydantic import Field

from .base_model import BaseModel
from .fragments import ResultJsonElementTypeFields


class EvaluateTestChecks(BaseModel):
    evaluate_test_checks: Optional["EvaluateTestChecksEvaluateTestChecks"] = Field(
        alias="evaluateTestChecks"
    )


class EvaluateTestChecksEvaluateTestChecks(BaseModel):
    result_json: List["EvaluateTestChecksEvaluateTestChecksResultJson"] = Field(
        alias="resultJson"
    )


class EvaluateTestChecksEvaluateTestChecksResultJson(ResultJsonElementTypeFields):
    pass


EvaluateTestChecks.model_rebuild()
EvaluateTestChecksEvaluateTestChecks.model_rebuild()
