fragment QuestionAnswerPairFragment on QuestionAnswerPairType {
  id
  inputUnderTest
  llmOutput
  context
  outputContext
  metadata {
    inTokens
    outTokens
    durationSeconds
  }
  fileIds
  localEvals {
    id
    score
    feedback
    createdAt
  }
  test {
    id
  }
  status
}
