fragment RunFragment on RunType {
  qaSet {
    id
  }
  runId
  runReview {
    id
  }
  status
  textSummary
  timestamp
  completedAt
  archived
  name
  errorMessage
  averageDuration
  averageTokensIn
  averageTokensOut
  parameters {
    evalModel
    maximumThreads
    runConfidenceEvaluation
    heavyweightFactor
    createTextSummary
    modelUnderTest
    temperature
    maxOutputTokens
    systemPrompt
    customParameters
  }
  passRate {
    value
    error
  }
  successRate {
    value
    error
  }
  customMetrics {
    metricId
    name
    passRate
    error
  }
  testSuite {
    id
    title
  }
  project {
    id
    slug
  }
}
