# Generated by ariadne-codegen
# Source: vals/graphql/

from typing import List

from pydantic import Field

from .base_model import BaseModel
from .fragments import QuestionAnswerPairFragment


class ListQuestionAnswerPairs(BaseModel):
    question_answer_pairs_with_count: (
        "ListQuestionAnswerPairsQuestionAnswerPairsWithCount"
    ) = Field(alias="questionAnswerPairsWithCount")


class ListQuestionAnswerPairsQuestionAnswerPairsWithCount(BaseModel):
    question_answer_pairs: List[
        "ListQuestionAnswerPairsQuestionAnswerPairsWithCountQuestionAnswerPairs"
    ] = Field(alias="questionAnswerPairs")
    count: int


class ListQuestionAnswerPairsQuestionAnswerPairsWithCountQuestionAnswerPairs(
    QuestionAnswerPairFragment
):
    pass


ListQuestionAnswerPairs.model_rebuild()
ListQuestionAnswerPairsQuestionAnswerPairsWithCount.model_rebuild()
