# Generated by ariadne-codegen
# Source: vals/graphql/

from .base_model import BaseModel
from .fragments import RunFragment


class PullRun(BaseModel):
    run: "PullRunRun"


class PullRunRun(RunFragment):
    pass


PullRun.model_rebuild()
