#import QuestionAnswerPairFragment from "../fragments/qa_sets.graphql"

query ListQuestionAnswerPairs($qaSetId: String!, $offset: Int, $limit: Int) {
  questionAnswerPairsWithCount(
    qaSetId: $qaSetId
    filterOptions: { offset: $offset, limit: $limit }
  ) {
    questionAnswerPairs {
      ...QuestionAnswerPairFragment
    }
    count
  }
}
